#!/usr/bin/perl

# ATTTTTTTTTTTTTTCTTG

$|=1;
use strict;
use Async;

use Time::HiRes qw( gettimeofday tv_interval );

use ePOSSUM;
my $output=new ePOSSUM;

my $link=shift;
my $tempdir='/data/tmp/';
my $html_file= $tempdir.$link.'.html';
my $html_target= '/temp/'.$link.'.html';
my $data_file= $tempdir.$link.'.data';



open (IN,'<',$data_file) || die ($!);
my $tfs=(<IN>);
$output->PegOut ("internal error") unless $tfs=~s/^TF:\s+//;
chomp $tfs;
my @variants=();
while (<IN>) {
	chomp;
	my %data=();
	my ($wt,$mt,$variant)=split /\t/;
	if ($variant) {
		if ($variant=~/^(\d+):(\d+)(.*?)>(.*)/) {  #/
			$data{variant}=$variant;
			@data{qw /chr start old new/}=($1,$2,$3,$4);
			$data{name}=join ("_",$1,$2,$3,$4);
			push @variants,\%data;
		}
	}
	elsif ($mt || $wt) {
		@data{qw /wt mt name/}=($wt,$mt,'var'.($.-1));
		push @variants,\%data;
	}
}
close IN;

my $variants2analyse=scalar @variants;

my $started=0;
my $running=0;
my $completed=0;
my $cycle=1;
while ($variants2analyse>$completed) {
	$output->StartOutput("TFBS batch analysis running...",	{		refresh  => [ $html_target, 16 ],		filename => $html_file	});
	print qq !Selected TFs: $tfs<br>\n!;
	print "cycle ",$cycle++,": $variants2analyse variants to analyse, $started started, $completed completed...<hr>";
	print '<table border="0" cellpadding="4"><tr><td>number</td><td>name</td><td>wild-type seq</td><td>variant seq</td><td>variant</td><td>results HTML</td><td>results table</td></tr>';
	my $line=1;
	my $notes='';
	foreach my $dataref (@variants) {
		$dataref->{html}='/temp/'.$link.'_'.$dataref->{name}.'.html' unless $dataref->{html};
		$dataref->{table}='/temp/'.$link.'_'.$dataref->{name}.'.txt' unless $dataref->{table};
		$dataref->{tablefile}=$tempdir.$link.'_'.$dataref->{name}.'.txt' unless $dataref->{tablefile};
		$dataref->{errorfile}=$tempdir.$link.'_'.$dataref->{name}.'.fail' unless $dataref->{errorfile};
		unless ($dataref->{status} and $running < 20) {
			my $string='html_output='.$link.'_'.$dataref->{name}.'.html tfs='.$tfs;
			foreach my $key (qw /name chr start old new wt mt/) { #/
				$string.=" $key=$dataref->{$key}";
			}
			$dataref->{proc} = AsyncTimeout->new(sub {exec ("/usr/bin/perl /www/ePOSSUM/cgi-bin/PredictTFBSquot.pl ".$string)},220);
			$dataref->{status}='running';
			$running++;
			$started++;
		}
		elsif ($dataref->{status} eq 'running') {
			if (-e $dataref->{tablefile}) {
				$dataref->{status}='completed';
				$completed++;
				$running--;
			}
			elsif (-e $dataref->{errorfile}) {
				$dataref->{status}='<div style="color: red">failed</DIV>';
				$completed++;
				$running--;
			}
		}
		elsif ($dataref->{status} ne 'completed' and $dataref->{proc}->ready and $dataref->{proc}->error) {
			$dataref->{status}=$dataref->{proc}->error;
			$completed++;
			$running--;
		}
		print "<tr><td>",join ("</td><td>",$line,@{$dataref}{qw !name wt mt variant!},qq !<A HREF="$dataref->{html}">$dataref->{status}</A>!,$dataref->{status}),"</td></tr>\n";
		$line++;
	}
	print "</table>";
	print "sleeping for 10 seconds... $notes";
	print "</body></html>";
	sleep 10;
	last if $cycle>50;
}	
if ($cycle>50) {
	$output->StartOutput("TFBS batch analysis stopped (time-out)",	{			filename => $html_file	});
}
else {
	$output->StartOutput("TFBS batch analysis completed.",	{			filename => $html_file	});
}
print '<table border="0" cellpadding="4"><tr><td>number</td><td>name</td><td>wild-type seq</td><td>variant seq</td><td>variant</td><td>results HTML</td><td>results table</td></tr>';
my $line=1;
foreach my $dataref (@variants) {
	print "<tr><td>",join ("</td><td>",$line,@{$dataref}{qw !name wt mt variant!},
		qq !<A HREF="$dataref->{html}">$dataref->{status}</A>!,qq !<A HREF="$dataref->{table}">$dataref->{status}</A>!),"</td></tr>\n";
	$line++;
}
print "</table>";
$output->EndOutput();
exit 0;


__END__

my %out;
my %FORM=$cgi->Vars();

foreach my $key (keys %FORM) {
	$FORM{$key}=~s/^\s//;
	$FORM{$key}=~s/\s$//;
}

$FORM{name}='test';
my $filename="ePOSSUM_temp_".($FORM{name}?$FORM{name}:'').time()."_".int(rand(100000));
$out{html_output}=$filename.'.html';
my $html_target='/temp/'.$out{html_output};

$out{tfs}=($FORM{all_tf}?'all':join (",",grep s/^tf_//, keys %FORM));
my @errors=();

$FORM{coordinates}=~s/^Chr//i;
$FORM{coordinates}=~s/^X:/23/;
$FORM{coordinates}=~s/^Y:/24/;
$FORM{coordinates}=~s/^MT*:/0/;
push @errors,("No sequences or genomic positions entered") unless $FORM{sequences} or $FORM{coordinates};
push @errors,("We can only handle 100 positions / sequences pairs at one time") if (length $FORM{sequences}>1000 or length $FORM{coordinates}>1000);
push @errors,("If you want to analyse position and sequence, please use the 'variant' field.") if ($FORM{coordinates} and ($FORM{sequences}) ) ;
if ($FORM{'known_tfbs'}) {
	push @errors,("You must make up your mind - ePOSSUM can scan either known TFBS <i>or</i> TFBS for selected TFs.") if $out{tfs} ;
	push @errors,("ePOSSUM cannot scan known TFBS at a position without knowing the position, can we?") unless $FORM{coordinates} ;
	$out{tfs}='known';	
}
push @errors,("No TF(s) selected!") unless $out{tfs};
# die ($out{tfs});

if ($FORM{coordinates}) {
#	if ($FORM{coordinates}=~/^(\d+):(\d+)-(\d+)/){ 
#		@out{qw /chr start end/}=($1,$2,$3);  #/
#	}
	if ($FORM{coordinates}=~/^(\d+):(\d+)(.*?)>(.*)/) {  #/
		@out{qw /chr start old new/}=($1,$2,$3,$4);
	}
	else {
		push @errors,("Genomic position not in the correct format: ". $FORM{coordinates});
	}
	#delete $FORM{coordinates};
}

$out{wt}=uc $FORM{wildtype};
$out{mt}=uc $FORM{variant};

push @errors,("Illegal bases in wild-type sequence: $out{wt}") if $out{wt} and  $out{wt}!~/^[ACGT]+$/;
push @errors,("Illegal bases in variant sequence: $out{mt}") if $out{mt} and  $out{mt}!~/^[ACGT]+$/;
$output->PegOut('Cannot commence search',@errors) if @errors;
$output->StartOutput("Search for TFBSs...",{refresh=>[$html_target,5],});
#print "Content-Type: text/html\n\n";

my $cmd="/usr/bin/perl /www/ePOSSUM/cgi-bin/PredictTFBSquot.pl ".join (" ",map {"$_=$out{$_}"} keys %out);
print qq !Selected TFs: $out{tfs}<br>\n!;
print qq !Wild-type sequence: $out{wt}<br>\n!;
print qq !Variant sequence: $out{mt}<br>\n!;
print qq !Genomic position: !.$FORM{coordinates}."<br>\n";
print qq !<h3 class="red">DON'T TRY TO RELOAD THIS PAGE, USE THE HYPERLINK BELOW INSTEAD</h3>\n!; #'
print qq !<A HREF="$html_target">see status</A>\n!;
#print "$cmd<br>";
print "</body></html>\n\n";

close (STDOUT);
close (STDIN);
undef $output;


system ($cmd);

exit 0;